/* For modern browsers */
.clearfix:before,
.clearfix:after {
  content: "";
  display: table; }

.clearfix:after {
  clear: both; }

.clearfix {
  zoom: 1; }

/* For IE 6/7 (trigger hasLayout) */
.blog_date {
  font-family: open_sans;
  font-weight: bold;
  font-style: italic;
  color: #b3b3b3;
  margin-bottom: 10px;
  text-shadow: 0px -1px 0px rgba(50, 50, 50, 0.6); }

#inner_blog_navigation {
  border-top: 1px solid #ababab;
  margin-top: 10px;
  padding-top: 10px; }

#blog_nav_separator {
  text-align: center;
  width: 20px;
  float: left; }

#inner_blog_navigation_prev {
  float: left; }

#inner_blog_navigation_next {
  float: right; }

.blog-right-nav-arrow {
  vertical-align: middle;
  margin-left: 10px; }

.blog-left-nav-arrow {
  vertical-align: middle;
  margin-right: 10px; }

article.blog_post img {
  box-shadow: 0px 0px 10px #000;
  margin: 5px 0px 5px 0px; }
article.blog_post img.pulled-left {
  margin-right: 10px; }
article.blog_post img.pulled-right {
  margin-left: 10px; }
article.blog_post .col-2 > div {
  width: 49%;
  float: left; }
article.blog_post .col-2 > div:first-of-type {
  margin-right: 1%; }
article.blog_post .col-2 > div:last-of-type {
  margin-left: 1%; }
article.blog_post .col-3 > div {
  width: 32%;
  float: left;
  background-color: #aaa; }
article.blog_post .col-3 > div:nth-of-type(2) {
  margin-left: 1%;
  margin-right: 1%; }
